"""Top-level package for eve-jwt."""

__author__ = """Yossi Mosbacher"""
__email__ = 'joe.mosbacher@gmail.com'
__version__ = '0.1.9'

from .validation import AsymmetricKeyValidator
from .auth import JWTAuth
